#include <libfilezilla/format.hpp>

#include "strresult.hpp"

namespace fz {

std::string_view strresult(result r)
{
	using namespace std::string_view_literals;

	switch (r.error_) {
		case result::noperm:         return "Permission denied"sv;
		case result::nofile:         return "Couldn't open the file"sv;
		case result::nodir:          return "Couldn't open the directory"sv;
		case result::other: {
			switch (r.raw_) {
				case FZ_RESULT_RAW(ERROR_CALL_NOT_IMPLEMENTED, ENOSYS):
					return "Operation not implemented"sv;

				case FZ_RESULT_RAW(ERROR_ALREADY_EXISTS, EEXIST):
					return "File or directory already exists"sv;

				case FZ_RESULT_RAW(ERROR_TIMEOUT, ETIMEDOUT):
					return "Operation has timed out"sv;
			}

			return "Unknown error"sv;
		}

		case result::ok:             return "No error"sv;
		case result::nospace:        return "No space left"sv;
		case result::invalid:        return "Invalid file name or path"sv;
		case result::resource_limit: return "Too many open files or directories"sv;
	}

	return {};
}

}
